import es
import gamethread

# Register the addon with EventScripts
info = es.AddonInfo()
info.name     = "TimeOut!"
info.version  = "1.3"
info.url      = "http://forums.mattie.info/cs/forums/viewtopic.php?t=5360"
info.basename = "timeout"
info.author   = "XE_ManUp & Mattie"

dict_timeoutPlayers     = {}
dict_savedTimeOuts      = {}
list_currentTimeOuts    = []

class PlayerSetUp:
    "Class that contains the data for resetting players"
    int_roundsLeft  =   0
    int_ffCount     =   0
    int_holdToggle  =   0

def load():
    # Set up a public variable for version information
    es.set('xe_timeout', '1.3')
    es.makepublic('xe_timeout')
    
    # Set the variables used by TimeOut
    es.ServerVar('timeout_rounds').set(0)
    es.ServerVar('timeout_sounds').set(0)
    es.server.cmd('exec timeout/timeout.cfg')
    
    # See if we need to add the sounds to the stringtables for download
    if (es.ServerVar('timeout_sounds') == '1'):
        es.stringtable('downloadables', 'sound/timeout/timeout1.mp3')
        es.stringtable('downloadables', 'sound/timeout/timeout2.mp3')
        es.stringtable('downloadables', 'sound/timeout/timeout3.mp3')
        es.stringtable('downloadables', 'sound/timeout/timeout4.mp3')
        
    # Add all current players to the dict_timeoutPlayers dictionary
    for userid in es.getUseridList():
    
        # Let's skip adding BOTs to the dictionary
        if 'BOT' not in es.getplayersteamid(userid):
            dict_timeoutPlayers[str(userid)] = PlayerSetUp()
            
def unload():
    dict_timeoutPlayers.clear()
    dict_savedTimeOuts.clear()
    list_currentTimeOuts = []
    
def es_map_start(event_var):
    # Clear/reset the timeout dictionaries
    dict_timeoutPlayers.clear()
    dict_savedTimeOuts.clear()
    
    # See if we need to add the sounds to the stringtables for download
    if (es.ServerVar('timeout_sounds') == '1'):
        es.stringtable('downloadables', 'sound/timeout/timeout1.mp3')
        es.stringtable('downloadables', 'sound/timeout/timeout2.mp3')
        es.stringtable('downloadables', 'sound/timeout/timeout3.mp3')
        es.stringtable('downloadables', 'sound/timeout/timeout4.mp3')

def player_activate(event_var):
    userid = event_var['userid']
    # Let's skip adding BOTs to the dictionary
    if event_var['es_steamid'] != 'BOT':
        dict_timeoutPlayers[userid] = PlayerSetUp()
        
        es.msg(dict_timeoutPlayers)
        
        # See if they tried reconnecting to try to "cheat the system"
        if dict_savedTimeOuts.has_key(event_var['es_steamid']):
            # Set the player's timeout rounds to the appropriate number
            dict_timeoutPlayers[userid].int_roundsLeft = dict_savedTimeOuts[event_var['es_steamid']]
            
            # Add their userid from the list of current timeouts
            list_currentTimeOuts.append(userid)

def player_disconnect(event_var):
    # Let's skip this check for BOTs since they are not in the dictionary
    if event_var['es_steamid'] != 'BOT':
        # Let's see if we need to save their timeout rounds to the dict_savedTimeOuts
        if dict_timeoutPlayers[event_var['userid']].int_roundsLeft > 0:
            dict_savedTimeOuts[event_var['es_steamid']] = dict_timeoutPlayers[event_var['userid']].int_roundsLeft
            
            # Remove their userid from the list of current timeouts
            list_currentTimeOuts.remove(userid)
            
        # Remove the player from the dict_timeoutPlayers
        del dict_timeoutPlayers[event_var['userid']]

def player_hurt(event_var):
    attacker = event_var['attacker']
    # See if the attacker's team is the same as the victim's team
    if event_var['es_attackerteam'] != event_var['es_userteam']:
        return
        
    # Make sure that the victim's userid isn't the attacker's userid (we don't want to punish the player for hurting themselves
    if event_var['userid'] == event_var['attacker']:
        return
        
    # Make sure we don't punish BOTs...this would be pretty much pointless
    if event_var['es_attackersteamid'] == 'BOT':
        return
        
    '''
        Shotguns and grenades are evil for this script. I have created (with
        the help of Mattie) a "hold toggle" specifically for these since they
        either do multiple damage in 1 hit (shotguns), or can damage more than
        one person in 1 hit(grenades).
    '''
    
    # Make sure that the "hold toggle" is not active:
    if dict_timeoutPlayers[attacker].int_holdToggle:
        return
        
    # Set the "hold toggle" to active
    dict_timeoutPlayers[attacker].int_holdToggle = 1
    
    # Set up a delay to set the hold toggle to inactive
    gamethread.delayed(0.2, resetHoldToggle, (attacker))
    
    # Increment the friendly fire count
    dict_timeoutPlayers[attacker].int_ffCount += 1
    
    # Send the appropriate message or start the timeout
    if dict_timeoutPlayers[attacker].int_ffCount == 1:
        es.tell(attacker, '\x04[TimeOut]\x01 That\'s\x03 1\x01 ...do you want to go to timeout?')
    elif dict_timeoutPlayers[attacker].int_ffCount == 2:
        es.tell(attacker, '\x04[TimeOut]\x01 That\'s\x03 2\x01 ...do you want to go to timeout?')
    elif dict_timeoutPlayers[attacker].int_ffCount == 3:
        # Send the player to the spectator team --- AKA "TimeOut"
        es.changeteam(attacker, 1)
        
        # Add their userid from the list of current timeouts
        list_currentTimeOuts.append(attacker)
        
        # Set the number of timeout rounds for the player
        dict_timeoutPlayers[attacker].int_roundsLeft = int(es.ServerVar('timeout_rounds'))
        
        # Let the other player's know that the attacker was sent to timeout
        if es.ServerVar('timeout_rounds') == '1':
            es.msg('\x04[TimeOut]\x03 %s\x01 has been placed in timeout for\x03 1\x01 round for team-wounding.' %event_var['es_attackername'])
        else:
            es.msg('\x04[TimeOut]\x03 %s\x01 has been placed in timeout for \x03%s\x01 rounds for team-wounding.' %(event_var['es_attackername'], es.ServerVar('timeout_rounds')))
    
    # See if sounds are active
    if (es.ServerVar('timeout_sounds') == '0'):
        return
        
    # Play the appropriate sound
    if dict_timeoutPlayers[attacker].int_ffCount == 1:
        es.playsound(attacker, 'timeout/timeout1.mp3', 1.0)
    elif dict_timeoutPlayers[attacker].int_ffCount == 2:
        es.playsound(attacker, 'timeout/timeout2.mp3', 1.0)
    elif dict_timeoutPlayers[attacker].int_ffCount == 3:
        es.playsound(attacker, 'timeout/timeout3.mp3', 1.0)

def player_team(event_var):
    userid = event_var['userid']
    
    # We don't worry about it if they are joining spectator
    if int(event_var['team']) == 1:
        return
        
    # We don't worry about BOTs either
    if es.isbot(userid):
        return
        
    # When I run the checkTimeOut(), I decrease the count, so I need to add one to make this accurate
    if userid not in list_currentTimeOuts:
        return
        
    # Check to see if they have any remaining timeout left
    roundsRemaining = dict_timeoutPlayers[userid].int_roundsLeft
    
    # For display purposes, I need to check and see if roundsRemaining is > 0
    if roundsRemaining == 0:
        roundsRemaining = roundsRemaining + 1
        
    # Send the player back to spectator ... so desperate
    gamethread.delayed(1, es.changeteam, (userid, 1))
    
    # Send the player a fun message telling them to chill out and wait until their timeout is over
    if roundsRemaining == 1:
        gamethread.delayed(1, es.tell, (userid, '\x04[\x03TimeOut\x04]\x01 You have\x03 1 \x01more round until your timeout is over.'))
    else:
        gamethread.delayed(1, es.tell, (userid, '\x04[\x03TimeOut\x03]\x01\x01 You have\x03 %s \x01more rounds until your timeout is over.' %roundsRemaining))

def round_start(event_var):
    for userid in dict_timeoutPlayers:
        # Reset the friendly fire count for this round
        dict_timeoutPlayers[userid].int_ffCount = 0
        
        # See if they are currently in timeout
        if userid in list_currentTimeOuts:
            # Run a check to see if we can remove them from timeout
            checkTimeOut(userid)

def checkTimeOut(userid):
    # Check to see if they have any remaining timeout left
    roundsRemaining = dict_timeoutPlayers[userid].int_roundsLeft
    if roundsRemaining < 1:
        es.tell(userid, '\x04[\x03TimeOut\x04]\x01 Your timeout time is now over.')
        
        # See if we play a sound announcing that they can rejoin the game
        if es.ServerVar('timeout_sounds') == '1':
            es.playsound(userid, 'timeout/timeout4.mp3', 1.0)
        
        # Remove them from the current timeout list
        list_currentTimeOuts.remove(userid)
        
        # Place the player back on a random team
        gamethread.delayed(1, es.changeteam, (userid, 0))
        
    else:
        # Tell the player how many more rounds they have left in TimeOut
        if roundsRemaining == 1:
            gamethread.delayed(1, es.tell, (userid, '\x04[TimeOut]\x01 You have\x03 1\x01 more round until your timeout is over.')) 
        else:
            gamethread.delayed(1, es.tell, (userid, '\x04[TimeOut]\x01 You have\x03 %s \x01more rounds until your timeout is over.' %roundsRemaining))
            
        # Update the rounds remaining in timeout (subtract 1)
        dict_timeoutPlayers[userid].int_roundsLeft -= 1

def resetHoldToggle(userid):
    # Make sure the player didn't disconnect in the 0,2 second delay
    if dict_timeoutPlayers.has_key(userid):
        # Set the "hold toggle" to inactive
        dict_timeoutPlayers[userid].int_holdToggle = 0